from distutils.core import setup

setup(name='pygeotemporal',
      version='0.1.0',
      url='https://opensource.ncsa.illinois.edu/bitbucket/projects/GEOD/repos/pygeotemporal/browse',
      packages=['pygeotemporal'],
      )
